var gulp   = require('gulp');
var Elixir = require('laravel-elixir');
var _ = require('underscore');

var config = Elixir.config;
var $      = Elixir.Plugins;
var template = require('gulp-template-compile');

/**
 * Compile templates
 */
Elixir.extend('templates', function (src, output, options) {
    var paths = prepGulpPaths(src, output);
    var pluginOptions = _.extend(config.templates.underscore.pluginOptions, options);

    new Elixir.Task('templates', function () {
        this.log(paths.src, paths.output);

        return (gulp
            .src(paths.src.path)
            .pipe($.if(config.sourcemaps, $.sourcemaps.init()))
            .pipe(template(pluginOptions))
            .pipe($.concat(paths.output.name))
            .pipe($.if(config.sourcemaps, $.sourcemaps.write('.')))
            .pipe(gulp.dest(paths.output.baseDir))
            .pipe(new Elixir.Notification('Templates Compiled!'))
        );
    })
        .watch(paths.src.path);
});

/**
 * Prep the Gulp src and output paths.
 *
 * @param  {string|array} src
 * @param  {string|null}  output
 * @return {object}
 */
var prepGulpPaths = function(src, output) {
    return new Elixir.GulpPaths()
        .src(src, config.get('assets.templates.underscore.folder'))
        .output(output || config.get('public.templates.underscore.outputFolder'), 'templates.js');
};